## About Healthcare/IO Parser

The Healthcare/IO **parser** is an Electronic Data Interchange (EDI) parser developed at Vanderbilt University Medical Center during Khanhly Nguyen's summer internship 2019. Built in a healthcare setting, the parser focuses (for now) on x12 claims (837) and remittances (835)

This code is intended to extract x12 837 and 835 and format them into portable and human readable format (JSON). This allows the claims to be stored in document data stores such as Mongodb, couchdb or databases that have support for JSON like PostgreSQL

We wrote this frame to be used in both command line or as a library within in your code. The framework is driven by configurations that derviced from X12 standards.

## Features

| Features | Description|
| -------- | --- |
|X12 claims/remits| parsing of {x12} claims/remittances into JSON format with human readible attributes|
|Multi Processing| capable of processing multiple files simultaneously to speed up processing|
|Process Recovery| capable of recovering interrupted runs|
|Export to RDBMS| exports data to relational format (NoSQL -> SQL) 8 supported industry standard databases <br> * PostgreSQL,<br> * Redshift, <br>* Neteeza, <br>* Mysql, <br>* Mariadb, <br>* bigquery, <br>* sqlite3<br>*databricks |
|**Issues and Bug reports**| info@the-phi.com


For advanced features visit [Healthcare/IO::Parser](https://healthcareio.the-phi.com/parser) homepage


## Installation

**System requirements**

    - git (latest cli)
    - python3+ (with virtualenv)
    - mongodb (optional)

**Installation command**

    pip install --upgrade git+https://hiplab.mc.vanderbilt.edu/git/hiplab/parser.git

## Usage 

Healthcare/IO is primarily intended to be used as a command line parser (for now). However it can be used as a library that you integrate into custom code. It is fully written in python 3+ under MIT License

**Learning More**

- More can be found at https://healthcareio.the-phi.com/ 
- The source code & example code are at https://hiplab.mc.vanderbilt.edu/git/hiplab/parser

**Known Limitations**

1. By default it does NOT come with all {X12} Segments. 
2. Renaming attributes requires writing plugins
3. Upgrade configuration may require dropping tables
4. For now can only read {x12} from disk (or s3 bucket)

There is support for additional features and attributes available at [Healthcare/IO::Parser](https://healthcareio.the-phi.com/parser).


## Credits

This project was realized thanks largely to the effort of  Vanderbilt University Medical Center 

|  | | &nbsp;|
| --- | --- | --- |
|- Khanhly Nguyen<br>- Heather Dunn<br>- Nina Thousand|- Byun Kang<br>- Gaylon Stanley<br>- Cheng Gao|- Thomas Brown<br>- Brad Malin<br>- Steve Nyemba



